      subroutine cmyEfield(aTrack, Efout)
!      This is a template for giving electric field which is
!      complex and   cannot  be expressed by the default method.
!      This should be copied to your application area and
!      modified as you like. For compilaton, you may simply
!      change a line in chook.mk: i.e
!       objs = chook.o
!      to 
!       objs = chook.o cmyEfield.o

!     Before making your application, the library must be
!     updated; For that
!       Define MYEFIELD  in Cosmos/cosmos/Zcondc.h
!       remove all in Cosmos/lib/$ARCH/
!       make veryclean
!       make
!     For actual run, you must specify HowEfield=2 in the
!     namelist ($HPARAM part).
!
      implicit none
#include "Zglobalc.h"
#include "Ztrack.h"
#include "Zobs.h"
#include "Zobsv.h"
#include "Zincidentv.h"
      record /track/ aTrack  ! input. Track info in E-xyz.  
       ! position and "where" info (or others ) are used.      
      real(8),intent(out):: Efout(3) ! Elecric field vector in
                 ! the E-xyz system. unit is  V/m.
!   This following information may be used to fix      Efout
!
!   particle position.
!    aTrack.pos.height: vertical height in m from the sealevel
!    aTrack.pos.depth:  depth in kg/m2.
!    aTrack.pos.xyz.r(1:3):  xyz pos in E-xyz system.
!                   to convert to other system,
!         see cdet2xyzFE.f and cxyz2det.f in Cosmos/Tracking.
!         Use routines cxyz2***.  *** depends on the requirment.
!         
!    aTrack.where: the observation level to which the ptcl
!                  is directed.     
!  The next observation point
!       ObsSites(aTrack.where).pos.height  ! height in m
!       ObsSites(aTrack.where).pos.depth  ! height in m
!  Deepest observation point
!       ObsSites(NoOfSites).pos.height
!       ObsSites(NoOfSites).pos.depth
!  distance r to the shower axis (perpendicular to the axis). r in m.
!       real(8)::r
!       call cgetPcoreDist(aTrack, r) 
!  distance r to the shower axis (horizontal to the axis). r in m
!       call cgetHcoreDist(aTrack, r)
!  time information 
!       aTrack.t/c*Tonsec  in ns. 
!      (c and Tonsec is deifined in Zglobalc.h; 0 is first col. 
!         point)
!  to convert Eelectric field (Ef) in detector or primary system
!      to the Exyz system. 
!     real(8):: Ef(3) 
!     call cdet2xyzD(Ef, Efout)  ! for detector to Exyz
!     call cprim2xyzD(Ef,Efout)  ! for primary to Exyz  
      real(8)::h
      real(8)::x,y
      real(8):: Efz, Ef(3)
      record /coord/ xyz

      h = aTrack.pos.height
      
      if(h > 7e3 ) then
         Efz = 0.
      else
!           convert x,y. origin is  at the deepest det. pos
         call cxyz2det( ObsSites(NoOfSites).pos.xyz,
     *            aTrack.pos.xyz, xyz)
         x = xyz.r(1)
         y = xyz.r(2)
!             E eists only in  some area
         if( x > 0. .and. y > 0.) then
            Efz= 360d3*sin(3.1415/2.*h/2d3)*exp(-h/5.d3)
         else
            Efz = 0.
         endif
      endif
      Ef(:) =(/0.d0, 0.d0, Efz/)
      call  cdet2xyzD(Ef, Efout)
      end subroutine cmyEfield
